/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "kdarcontroller.h"
#include "kdarListViewItem.h"

#include "kdarview.h"
#include "archiveHandle.h"

#include "createArchiveThread.h"
#include "diffArchiveThread.h"
#include "getUserResponseThread.h"
#include "isolateArchiveThread.h"
#include "listArchiveThread.h"
#include "openArchiveThread.h"
#include "populateTreeArchiveThread.h"
#include "restoreArchiveThread.h"
#include "testArchiveThread.h"

#include "kdarArchiveBrowserUpdatesEvent.h"
#include "kdarCancelButtonEvent.h"
#include "kdardeletethreadevent.h"
#include "kdarLogFileCommandEvent.h"
#include "kdarPostListViewItemEvent.h"
#include "kdarProgressEvent.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarTarListingEvent.h"
#include "kdarUserInteractionEvent.h"
#include "kdarWarningEvent.h"

#include "kdarConfig.h"

#include <kcursor.h>
#include <kdebug.h>
#include <kfiledialog.h>

#include <qapp.h>
#include <qevent.h>
#include <qwidget.h>

#include <dar/libdar.hpp>

extern bool libdarWorking;
extern kdarListViewItem* currentDirectory;
extern libdar::archive *theArchive;

KDarController::KDarController()
    : m_userResponseThread( new getUserResponseThread( this ) )
{
    //Initialize the thread pointers to zero:
    m_createThread = 0;
    m_diffThread = 0;
    m_isolateThread = 0;
    m_listThread = 0;
    m_openThread = 0;
    m_populateTreeThread = 0;
    m_restoreThread = 0;
    m_testThread = 0;
    //Set the answer callback functions for libdar
    //for warning messages -- which just get sent to the archiveTextBrowser
    libdar::set_warning_callback( &getUserResponseThread::interaction_warning_display );
    //for user interaction (yes/no responses) -- calls a
    //KMessageBox::warningYesNo
    libdar::set_answer_callback( &getUserResponseThread::interaction_pause );
    //send the tar output to the archiveListView:
    libdar::set_op_tar_listing_callback( &getUserResponseThread::tarListing );

    m_userResponseThread->start();

}

KDarController::~KDarController()
{
    if ( m_populateTreeThread )
    {
        m_populateTreeThread->wait();
        delete m_populateTreeThread;
        m_populateTreeThread = 0;
    }
    if ( m_listThread )
    {
        m_listThread->wait();
        delete m_listThread;
        m_listThread = 0;
    }
    if ( m_testThread )
    {
        m_testThread->wait();
        delete m_testThread;
        m_testThread = 0;
    }
    if ( m_restoreThread )
    {
        m_restoreThread->wait();
        delete m_restoreThread;
        m_restoreThread = 0;
    }
    if ( m_isolateThread )
    {
        m_isolateThread->wait();
        delete m_isolateThread;
        m_isolateThread = 0;
    }
    if ( m_diffThread )
    {
        m_diffThread->wait();
        delete m_diffThread;
        m_diffThread = 0;
    }
    if ( m_createThread )
    {
        m_createThread->wait();
        delete m_createThread;
        m_createThread = 0;
    }
}

void KDarController::closeArchive()
{
    if ( theArchive )
    {
        delete theArchive;
        theArchive = 0;
        kdDebug() << "Archive closed." << endl;
    }
}

void KDarController::closeLogFile()
{
    //close the log file
    m_logFile.close();
    m_logStream.unsetDevice();
    kdDebug() << "KDarController::closeLogFile()" << endl;
}

void KDarController::createArchive( KDarConfig * kdarconfig, bool updateListView )
{
    if (libdarWorking)
    {
        return;
    }
    //parse the new archive name for string substitutions:
    kdarconfig->setNewArchiveName( archiveHandle::parseStringSubstitutions( kdarconfig->newArchiveName(), kdarconfig ) );

    kdDebug() << "KDarController::createArchive(): newArchiveName is " << kdarconfig->newArchiveName().latin1() << endl;
    if ( kdarconfig->differentialBackup() )
    {
        kdDebug() << "KDarController::createArchive(): opening reference archive" << endl;
        openArchive( kdarconfig, updateListView );
    }
    kdDebug() << "starting create thread." << endl;
    m_createThread = new createArchiveThread( this, kdarconfig );
    m_createThread->start();
    kdDebug() << "started create thread." << endl;
}

void KDarController::diffArchive( KDarConfig * kdarconfig )
{
    if (libdarWorking)
    {
        return;
    }
    if (theArchive == 0) return;
    QString diffArchiveDirectory = KFileDialog::getExistingDirectory(QString::null, 0, (QString) "Select directory to diff");
    //if user cancels, do nothing
    if ( diffArchiveDirectory.isEmpty() ) return;
    kdDebug() << "KDarContoller::diffArchive() called: directory to diff is " << kdarconfig->diffArchiveDirectory().latin1() << endl;
    kdarconfig->setDiffArchiveDirectory( diffArchiveDirectory );
    kdDebug() << "KDarController::diffArchive(): creating and starting m_diffThread" << endl;
    m_diffThread = new diffArchiveThread( this, kdarconfig, m_parent->archiveFileView );
    m_diffThread->start();

}

void KDarController::isolateArchive( KDarConfig * kdarconfig )
{
    if (libdarWorking)
    {
        return;
    }
    if (theArchive == 0) return;

    //user picks isolated archive name
    QString isoArchiveName=KFileDialog::getSaveFileName(QString::null, QString::null, 0, (QString) "Select a filename for new isolation archive");
    //if user cancels, do nothing
    if ( isoArchiveName.isEmpty() ) return;
    kdarconfig->setIsolationArchiveName( isoArchiveName );
    kdDebug() << "KDarContoller::isolateArchive() called: isolatation name is " << kdarconfig->isolationArchiveName().latin1() << endl;
    kdDebug() << "KDarController::isolateArchive(): creating and starting m_isolateThread" << endl;
    m_isolateThread = new isolateArchiveThread( this, kdarconfig );
    m_isolateThread->start();
}

void KDarController::listArchive( KDarConfig * kdarconfig, bool loadArchive )
{
    if (libdarWorking)
    {
        return;
    }
    if (theArchive == 0)
    {
        QString archiveName = kdarconfig->archiveName();
        if (archiveName[0] == '/' && loadArchive )
        {
            kdDebug() << "archive name to list is " << archiveName.latin1() << endl;
            openArchive( kdarconfig );
        }
        else
        {
            return;
        }
    }
    m_listThread = new listArchiveThread( this, kdarconfig );
    m_listThread->start();
    kdDebug() << "KDarContoller::listArchive() called." << endl;
}

void KDarController::openArchive( KDarConfig * kdarconfig, bool updateListView )
{
    if (libdarWorking)
    {
        return;
    }
    //Close the archive object if one is open
    if ( theArchive ) closeArchive();

    QString archiveName = kdarconfig->archiveName();
    if (archiveName[0] != '/')
    {
        return;
    }
    kdDebug() << "KDarContoller::openArchive() starting m_openThread." << endl;
    m_openThread = new openArchiveThread( this, kdarconfig, updateListView );
    m_openThread->start();
}

void KDarController::openLogFile( const QString & logFile )
{
    //open the log file
    if ( m_logFile.isOpen() ) m_logFile.close();
    m_logFile.setName( logFile );
    m_logFile.open( IO_WriteOnly | IO_Append );
    m_logStream.setDevice( (QIODevice *) &m_logFile );
    kdDebug() << "KDarController::openLogFile()" << endl;
}

void KDarController::populateArchiveTree( KDarConfig *, QListViewItem* directory )
{
    if (libdarWorking)
    {
        return;
    }
    waitForThread( controller::OPEN_THREAD );
    if (theArchive == 0) return;

    //Only populate this item if it is an empty directory
    if ( !directory ) return;
    if ( !directory->firstChild() ) return;
    if ( !( directory->firstChild()->text( 0 ) == "<empty>" ) ) return;
    //Otherwise we have an empty directory that needs to be populated
    currentDirectory = static_cast<kdarListViewItem*> (directory);
    kdDebug() << "KDarContoller::populateArchiveTree() called: creating and starting m_populateTreeThread" << endl;
    m_populateTreeThread = new populateTreeArchiveThread( this, directory );
    m_populateTreeThread->start();
}

void KDarController::restoreArchive( KDarConfig * kdarconfig )
{
    if ( libdarWorking )
    {
        return;
    }
    kdDebug() << "KDarContoller::restoreArchive() called." << endl;
    kdDebug() << "starting create thread." << endl;

    m_restoreThread = new restoreArchiveThread( this, kdarconfig, m_parent->archiveFileView );
    m_restoreThread->start();
    kdDebug() << "started create thread." << endl;
}

void KDarController::testArchive( KDarConfig * kdarconfig )
{
    if (libdarWorking)
    {
        return;
    }
    if (theArchive == 0) return;
    //test the archive
    kdDebug() << "KDarContoller::testArchive() creating and starting m_testThread" << endl;
    m_testThread = new testArchiveThread( this, kdarconfig, m_parent->archiveFileView );
    m_testThread->start();
}

void KDarController::customEvent( QCustomEvent * e )
{
    if ( e->type() == (QEvent::Type) KDar::SETCURSOR )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarSetCursorEvent* sce = (kdarSetCursorEvent*)e;
            kdarSetCursorEvent* newsce = new kdarSetCursorEvent( sce->cursor() );
            QApplication::postEvent( m_parent, newsce );
        }
    }
    if ( e->type() == ( QEvent::Type ) KDar::PROGRESS )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarProgressEvent *pe = (kdarProgressEvent*) e;
            kdarProgressEvent *newpe = new kdarProgressEvent( pe->command() );
            QApplication::postEvent( m_parent, newpe );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( e->type() == ( QEvent::Type ) KDar::STATUSBAR )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarStatusBarEvent *se = (kdarStatusBarEvent*) e;
            //Fix this output: There are spaces between lines.
//            QString message = se->message();
//            writeLogMessage( message );
            kdarStatusBarEvent *newse = new kdarStatusBarEvent( se->message(), se->duration() );
            QApplication::postEvent( m_parent, newse );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( e->type() == ( QEvent::Type ) KDar::WARNING )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarWarningEvent *we = (kdarWarningEvent*) e;
            QString message = we->message();
            writeLogMessage( message );
            kdarWarningEvent *newwe = new kdarWarningEvent( we->message() );
            QApplication::postEvent( m_parent, newwe );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( e->type() == ( QEvent::Type ) KDar::TARLISTING )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarTarListingEvent *tle = (kdarTarListingEvent*) e;
            kdarTarListingEvent *newtle = new kdarTarListingEvent( tle->directory(), tle->flag(), tle->perm(), tle->uid(), tle->gid(), tle->size(), tle->date(), tle->filename() );
            QApplication::postEvent( m_parent, newtle );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( e->type() == ( QEvent::Type ) KDar::USERINTERACTION )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarUserInteractionEvent *uie = (kdarUserInteractionEvent*) e;
            kdarUserInteractionEvent *newuie = new kdarUserInteractionEvent( uie->question() );
            QApplication::postEvent( m_parent, newuie );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( e->type() == ( QEvent::Type ) KDar::POSTLISTVIEWITEM )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarPostListViewItemEvent *pe = (kdarPostListViewItemEvent*) e;
            kdarPostListViewItemEvent *newpe = new kdarPostListViewItemEvent( pe->text(), pe->clearView() );
            QApplication::postEvent( m_parent, newpe );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( e->type() == ( QEvent::Type ) KDar::ARCHIVEBROWSERUPDATES )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarArchiveBrowserUpdatesEvent *ae = (kdarArchiveBrowserUpdatesEvent*) e;
            kdarArchiveBrowserUpdatesEvent *newae = new kdarArchiveBrowserUpdatesEvent( ae->command(), ae->item() );
            QApplication::postEvent( m_parent, newae );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( e->type() == ( QEvent::Type ) KDar::DELETETHREAD )
    {
        kdarDeleteThreadEvent* dte = (kdarDeleteThreadEvent*)e;
        QThread* thread = dte->thread();
        if ( !thread ) return;
        if ( thread->running() )
        {
            thread->wait();
        }
        delete thread;
        //set the corresponding thread pointer to zero
        if ( thread == m_createThread )
        {
            m_createThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_createThread" << endl;
        }
        if ( thread == m_diffThread )
        {
            m_diffThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_diffThread" << endl;
        }
        if ( thread == m_isolateThread )
        {
            m_isolateThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_isolateThread" << endl;
        }
        if ( thread == m_listThread )
        {
            m_listThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_listThread" << endl;
        }
        if ( thread == m_openThread )
        {
            m_openThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_openThread" << endl;
        }
        if ( thread == m_populateTreeThread )
        {
            m_populateTreeThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_populateTreeThread" << endl;
            //We need to re-enable updates to the archive browser:

        }
        if ( thread == m_restoreThread )
        {
            m_restoreThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_restoreThread" << endl;
        }
        if ( thread == m_testThread )
        {
            m_testThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_testThread" << endl;
        }
    }
    if ( e->type() == (QEvent::Type) KDar::CANCELBUTTON )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarCancelButtonEvent* cbe = (kdarCancelButtonEvent*)e;
            kdarCancelButtonEvent* newcbe = new kdarCancelButtonEvent( cbe->command() );
            QApplication::postEvent( m_parent, newcbe );
        }
    }
    if ( e->type() == (QEvent::Type) KDar::LOGFILECOMMAND )
    {
        kdarLogFileCommandEvent* lce = (kdarLogFileCommandEvent*)e;
        QString logText = lce->text();
        switch ( (int) lce->command() )
        {
            case kdarLogFileCommandEvent::OPEN:
                openLogFile( logText );
                break;
            case kdarLogFileCommandEvent::CLOSE:
                closeLogFile();
                break;
            case kdarLogFileCommandEvent::WRITEMESSAGE:
                writeLogMessage( logText );
                break;
            case kdarLogFileCommandEvent::FLUSH:
                m_logFile.flush();
            default:
                ;
        }
    }
}

void KDarController::waitForThread( controller::threadType type )
{
    //Check if the given thread type is running, and wait for it to finish
    switch ( type )
    {
        case controller::OPEN_THREAD:
            kdDebug() << "KDarController::waitForThread(): OPEN_THREAD" << endl;
            if ( m_openThread ) m_openThread->wait();
            break;
        case controller::CREATE_THREAD:
            if ( m_createThread ) m_createThread->wait();
            break;
        case controller::RESTORE_THREAD:
            if ( m_restoreThread ) m_restoreThread->wait();
            break;
        default:
            ;
    }
}

void KDarController::writeLogMessage( QString & message )
{
    QIODevice * device = m_logStream.device();
    if ( device )
    {
        if ( !message.isEmpty() )
        {
            QString msg = message;
            if ( !msg.endsWith( "\n" ) ) msg += "\n";
            kdDebug() << "KDarController::writeLogMessage(): writing " << msg.latin1();
            m_logStream << message;
            m_logFile.flush();
        }
        else
        {
            kdDebug() << "KDarController::writeLogMessage(): message \"" << message.latin1() << "\" is empty." << endl;

        }
    }
    else
    {
        kdDebug() << "KDarController::writeLogMessage(): m_logStream is not open." << endl;
        kdDebug() << "message was " << message.latin1();
    }
}

void KDarController::setParent( void *parent )
{
    kdDebug() << "KDarController::setParent(): setting m_parent." << endl;
    m_parent = static_cast<kdarView*>(parent);
    kdDebug() << "KDarController::setParent(): set m_parent." << endl;
}

#include "kdarcontroller.moc"
